unit TChartBugs;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VCLTee.TeeComma, VCLTee.TeEngine,
  Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.TeeTools, VCLTee.TeePoin3, VCLTee.Series,
  VCLTee.TeeOpenGL;

type
  TForm2 = class(TForm)
    PLSScoresChart: TChart;
    TeeCommanderScore: TTeeCommander;
    ChartTool1: TMarksTipTool;
    TeeOpenGL1: TTeeOpenGL;
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
   procedure FillTheScoreChart(TheChart: TChart; Color: TColor; Title: string; IsVisible: Boolean);
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

implementation

{$R *.dfm}

procedure TForm2.FillTheScoreChart(TheChart: TChart; Color: TColor; Title: string; IsVisible: Boolean);

procedure ApplyPCAChartSeriesSettings(Series: TPointSeries);
begin
  Series.Marks.Arrow.Color := 4194368;
  Series.Marks.Callout.Brush.Color := clBlack;
  Series.Marks.Callout.Draw3D := True;
  Series.Marks.Callout.Pen.Color := clDefault;
  Series.Marks.Callout.Visible := True;
  Series.Marks.Callout.Arrow.Color := 4194368;
  Series.Marks.Callout.ArrowHead := ahSolid;
  Series.Marks.Callout.ArrowHeadSize := 10;
  Series.Marks.Callout.Distance := 13;
  Series.Marks.Callout.Length := 38;
  Series.Marks.Font.Height := -19;
  Series.Marks.Frame.Visible := False;
  Series.Marks.ShapeStyle := fosRoundRectangle;
  Series.Marks.Transparent := True;
  Series.Marks.AutoPosition := False;
  Series.Marks.Visible := True;
  Series.LinePen.Visible := False;
end;

const
  DefaultPointerSizeC = 4;
var
  PointSeries: TPoint3DSeries;
  TheSeries: TPointSeries;
  OldCursor: TCursor;
  i,XAxisIndex,YAxisIndex,ZAxisIndex,CurIndex: Integer;
begin
  TheChart.Zoom.Pen.Color := clBlack;
  TheChart.AutoRepaint := False;
  OldCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  try
    TheChart.BottomAxis.Title.Caption := 'Factor-1';
    TheChart.LeftAxis.Title.Caption   := 'Factor-2';
    TheChart.DepthAxis.Title.Caption  := 'Factor-3';
    //
    PointSeries := TPoint3DSeries.Create(TheChart);
    PointSeries.BeginUpdate;
    PointSeries.Title := Title;
    PointSeries.ParentChart := TheChart;
    PointSeries.ColorEachPoint := False;
    PointSeries.SeriesColor := Color;
    PointSeries.Pointer.Dark3D := False;
    PointSeries.Pointer.InflateMargins := True;
    PointSeries.Pointer.Pen.Color := Color;
    PointSeries.Pointer.Style := psCircle;
    PointSeries.Pointer.HorizSize := DefaultPointerSizeC;
    PointSeries.Pointer.VertSize := DefaultPointerSizeC;
    PointSeries.Pointer.Depth := DefaultPointerSizeC;
    PointSeries.Pointer.Visible := IsVisible;
    PointSeries.XValues.Name := 'X';
    PointSeries.XValues.Order := loNone;
    PointSeries.YValues.Name := 'Y';
    PointSeries.YValues.Order := loNone;
    PointSeries.ZValues.Name := 'Z';
    PointSeries.ZValues.Order := loNone;
    TheSeries := TPointSeries(PointSeries);
    ApplyPCAChartSeriesSettings(TheSeries);
    try
      if Title = 'XScores' then
      begin
        PointSeries.AddXYZ( 0.1, 0.4, 0.9, 'Point_1_X', Color);
        PointSeries.AddXYZ(-0.5, 0.2, 0.3, 'Point_2_X', Color);
        PointSeries.AddXYZ(-0.1, 0.4, 0.7, 'Point_3_X', Color);
      end else
      begin
        PointSeries.AddXYZ(-0.10, 0.10, 0.45, 'Point_1_Y', Color);
        PointSeries.AddXYZ( 0.34, 0.15, 0.12, 'Point_2_Y', Color);
        PointSeries.AddXYZ(-0.24, 0.74, 0.67, 'Point_3_Y', Color);
      end;
    finally
      TheSeries.Marks.AutoPosition := False;
      TheSeries.Marks.Visible := True;
      TheSeries.EndUpdate;
      TheSeries.Active := IsVisible;
    end;
  finally
    TheChart.AutoRepaint := True;
    TheChart.Repaint;
    Screen.Cursor := OldCursor;
  end;
end;

procedure TForm2.FormShow(Sender: TObject);
begin
  PLSScoresChart.AutoRepaint := False;
  PLSScoresChart.BottomAxis.LabelsFont.Height := -16;
  PLSScoresChart.BottomAxis.LabelsFont.Style := [fsBold];
  PLSScoresChart.BottomAxis.Title.Font.Height := -13;
  PLSScoresChart.BottomAxis.Title.Font.Style := [fsBold];
  PLSScoresChart.DepthAxis.LabelsFont.Height := -16;
  PLSScoresChart.DepthAxis.LabelsFont.Style := [fsBold];
  PLSScoresChart.DepthAxis.Title.Font.Height := -13;
  PLSScoresChart.DepthAxis.Title.Font.Style := [fsBold];
  PLSScoresChart.LeftAxis.LabelsFont.Height := -16;
  PLSScoresChart.LeftAxis.LabelsFont.Style := [fsBold];
  PLSScoresChart.LeftAxis.Title.Font.Height := -13;
  PLSScoresChart.LeftAxis.Title.Font.Style := [fsBold];
  while PLSScoresChart.SeriesCount > 0 do
    PLSScoresChart.Series[0].Free;
  PLSScoresChart.AutoRepaint := True;
  FillTheScoreChart(PLSScoresChart, clBlue, 'XScores', True);
  FillTheScoreChart(PLSScoresChart, clRed, 'YScores', True);
  TeeCommanderScore.ButtonRotate.Down := True;
end;

end.
